#!/bin/sh

###############################################################################
#        NETWORK BACKUP SCRIPT FOR ENIGMA2/OPENVISION BACKUPSUITE             #
#                   MAKES A FULLBACK-UP TO MOUNTED NETWORK SHARE              #
###############################################################################

# Robust Python detection
detect_python() {
    if command -v python3 >/dev/null 2>&1; then
        echo "python3"
    elif command -v python2 >/dev/null 2>&1; then
        echo "python2"
    elif command -v python >/dev/null 2>&1; then
        echo "python"
    else
        echo "Unable to find Python!" >&2
        exit 1
    fi
}

# Terminal color setup
if tty >/dev/null 2>&1 ; then
    RED='-e \e[00;31m'
    GREEN='-e \e[00;32m'
    YELLOW='-e \e[01;33m'
    BLUE='-e \e[00;34m'
    PURPLE='-e \e[01;31m'
    WHITE='-e \e[00;37m'
else
    RED='\c00??0000'
    GREEN='\c0000??00'
    YELLOW='\c00????00'
    BLUE='\c0000????'
    PURPLE='\c00?:55>7'
    WHITE='\c00??????'
fi

# Library directory detection
if [ -d "/usr/lib64" ]; then
    LIBDIR="/usr/lib64"
else
    LIBDIR="/usr/lib"
fi

# Find Python and message script
PYTHON=$(detect_python)
MESSAGE_DIR="$LIBDIR/enigma2/python/Plugins/Extensions/BackupSuite"
MESSAGE_SCRIPT=""

# Find the best message script version
find_message_script() {
    # Check for compiled versions first
    for ext in pyc pyo; do
        if [ -f "$MESSAGE_DIR/message.$ext" ]; then
            echo "$MESSAGE_DIR/message.$ext"
            return
        fi
    done

    # Fallback to source version
    if [ -f "$MESSAGE_DIR/message.py" ]; then
        echo "$MESSAGE_DIR/message.py"
        return
    fi

    echo "Error: No message script found!" >&2
    exit 1
}

# Parameters
export LANG="${1:-en}"
export DEVICE_TYPE="${2:-NET}"
MEDIA="${3:-/media/net}"
export HARDDISK=0

# Find and validate message script
MESSAGE_SCRIPT=$(find_message_script)
export SHOW="$PYTHON $MESSAGE_SCRIPT $LANG"

# Show backup start message
# echo -n $GREEN
# $SHOW "message44" 2>&1  # Backup started...
echo -n $WHITE

# Create required directory structure
# Define the backup and image directories


BASE_BACKUP_DIR="$MEDIA/backup"
IMAGE_DIR="$BASE_BACKUP_DIR/image"

# Create directories if they don't exist
mkdir -p "$IMAGE_DIR" || {
    echo -n $RED
    $SHOW "message42e" 2>&1  # Failed to create backup directories!
    echo -n $WHITE
    exit 1
}


# Create marker file
touch "$BASE_BACKUP_DIR/backupstick" || {
    echo -n $RED
    $SHOW "message42f" 2>&1  # Failed to create backup marker!
    echo -n $WHITE
    exit 1
}

# Portable mount point validation
validate_network_path() {
    local path="$1"

    # Controllo semplificato: solo esistenza directory e permessi scrittura
    if [ ! -d "$path" ]; then
        $SHOW "message42a" 2>&1  # Directory non esiste!
        return 1
    fi

    # Verifica permessi scrivendo un file temporaneo
    local temp_test="$path/backup_test_$$.tmp"
    if ! touch "$temp_test" 2>/dev/null; then
        $SHOW "message42c" 2>&1  # Permessi scrittura negati!
        return 1
    fi
    rm -f "$temp_test"

    return 0
}

# Validate network path
if ! validate_network_path "$MEDIA"; then
    echo -n $RED
    $SHOW "message42" 2>&1  # Invalid network path!
    echo -n $WHITE
    exit 1
fi

# Calculate required space - portable method
calculate_space() {
    # Get used space in /usr
    if command -v du >/dev/null; then
        USEDSIZE=$(du -sk /usr 2>/dev/null | awk '{print $1}')
    else
        # Fallback to df if du not available
        USEDSIZE=$(df -k /usr | awk 'NR>1 {print $3}')
    fi

    # Calculate needed space with 20% buffer
    NEEDEDSPACE=$((USEDSIZE * 12 / 10))  # in KB

    # Get free space on target
    if command -v df >/dev/null; then
        FREESIZE=$(df -P -k "$MEDIA" 2>/dev/null | tail -1 | awk '{print $(NF-2)}')
    else
        FREESIZE=0
    fi

    echo "$USEDSIZE $NEEDEDSPACE $FREESIZE"
}

# Check available space
read USEDSIZE NEEDEDSPACE FREESIZE <<< $(calculate_space)

if [ -z "$FREESIZE" ] || [ "$FREESIZE" -lt "$NEEDEDSPACE" ]; then
    echo -n $RED
    $SHOW "message42d" 2>&1  # Insufficient free space!
    printf '%5s' "$((FREESIZE / 1024))" ; $SHOW "message32"  # Available (MB):
    printf '%5s' "$((NEEDEDSPACE / 1024))" ; $SHOW "message33"  # Needed (MB):
    echo -n $WHITE
    exit 1
fi

# Preparation message
echo -n $BLUE
$SHOW "message45" 2>&1  # Phase 1/3: Preparing backup environment
echo -n $WHITE

# Create version files
cp /etc/image-version "$BASE_BACKUP_DIR/imageversion" 2>/dev/null
cp /tmp/enigma2version "$BASE_BACKUP_DIR/enigma2version" 2>/dev/null

# Log NAS info
{
    echo "===== BackupSuite Network Backup ====="
    date
    echo "Backup directory: $BASE_BACKUP_DIR"
    echo "Image directory: $IMAGE_DIR"
    echo "Mount details:"
    grep "$MEDIA" /proc/mounts || echo "No mount info found"
    echo "Disk space:"
    df -h "$MEDIA"
    echo "======================================"
} > "$BASE_BACKUP_DIR/BackupSuite.log"

# Execute main backup script
BACKUP_SCRIPT="$LIBDIR/enigma2/python/Plugins/Extensions/BackupSuite/scripts/backupsuite.sh"
if [ -x "$BACKUP_SCRIPT" ]; then
    # Set executable permission
    chmod 755 "$BACKUP_SCRIPT" >/dev/null 2>&1

    # # Run backup
    # echo -n $GREEN
    # $SHOW "message46" 2>&1  # Creating backup image...
    echo -n $WHITE
    "$BACKUP_SCRIPT" "$BASE_BACKUP_DIR"
    ret=$?

    # Append backup log to our logfile
    if [ -f "$BASE_BACKUP_DIR/BackupSuite.log" ]; then
        cat "$BASE_BACKUP_DIR/BackupSuite.log" >> "$BASE_BACKUP_DIR/BackupSuite_full.log"
    fi
else
    echo -n $RED
    $SHOW "message47" 2>&1  # Backup script not found!
    echo -n $WHITE
    ret=1
fi

sync

# Completion message
if [ $ret -eq 0 ]; then
    echo -n $BLUE
    $SHOW "message48" 2>&1  # Backup completed successfully!

    # Calculate backup size
    SIZE=$(du -sh "$IMAGE_DIR" 2>/dev/null | awk '{print $1}')
    if [ -n "$SIZE" ]; then
        echo -n $YELLOW
        $SHOW "message49a" 2>&1  # Backup size:
        echo " $SIZE"
    fi

    # Show backup location
    echo -n $BLUE
    $SHOW "message50a" 2>&1  # Backup created in:
    echo " $IMAGE_DIR"

    # Clean up old backups (keep last 3)
    find "$MEDIA/backup" -maxdepth 1 -type d -name 'backup_*' 2>/dev/null |
        sort |
        head -n -3 |
        xargs rm -rf 2>/dev/null
else
    echo -n $RED
    $SHOW "message15" 2>&1  # Image creation FAILED!
    # Keep failed backup for debugging
fi

echo -n $WHITE
exit $ret
